@@ -32,7 +32,8 @@
 #include "unpack.h"
 #include "g_disptab.h"
 
-static GLint __glEvalComputeK(GLenum target)
+static GLint
+__glEvalComputeK(GLenum target)
 {
     switch (target) {
     case GL_MAP1_VERTEX_4:
@@ -41,33 +42,35 @@ static GLint __glEvalComputeK(GLenum target)
     case GL_MAP2_VERTEX_4:
     case GL_MAP2_COLOR_4:
     case GL_MAP2_TEXTURE_COORD_4:
-	return 4;
+        return 4;
     case GL_MAP1_VERTEX_3:
     case GL_MAP1_TEXTURE_COORD_3:
     case GL_MAP1_NORMAL:
     case GL_MAP2_VERTEX_3:
     case GL_MAP2_TEXTURE_COORD_3:
     case GL_MAP2_NORMAL:
-	return 3;
+        return 3;
     case GL_MAP1_TEXTURE_COORD_2:
     case GL_MAP2_TEXTURE_COORD_2:
-	return 2;
+        return 2;
     case GL_MAP1_TEXTURE_COORD_1:
     case GL_MAP2_TEXTURE_COORD_1:
     case GL_MAP1_INDEX:
     case GL_MAP2_INDEX:
-	return 1;
+        return 1;
     default:
-	return 0;
+        return 0;
     }
 }
 
-void __glXDispSwap_Map1f(GLbyte *pc)
+void
+__glXDispSwap_Map1f(GLbyte * pc)
 {
     GLint order, k;
     GLfloat u1, u2, *points;
     GLenum target;
     GLint compsize;
+
     __GLX_DECLARE_SWAP_VARIABLES;
     __GLX_DECLARE_SWAP_ARRAY_VARIABLES;
 
@@ -75,30 +78,33 @@ void __glXDispSwap_Map1f(GLbyte *pc)
     __GLX_SWAP_INT(pc + 12);
     __GLX_SWAP_FLOAT(pc + 4);
     __GLX_SWAP_FLOAT(pc + 8);
-    
-    target = *(GLenum *)(pc + 0); 
-    order = *(GLint *)(pc + 12);
-    u1 = *(GLfloat *)(pc + 4);
-    u2 = *(GLfloat *)(pc + 8);
-    points = (GLfloat *)(pc + 16);
+
+    target = *(GLenum *) (pc + 0);
+    order = *(GLint *) (pc + 12);
+    u1 = *(GLfloat *) (pc + 4);
+    u2 = *(GLfloat *) (pc + 8);
+    points = (GLfloat *) (pc + 16);
     k = __glEvalComputeK(target);
 
     if (order <= 0 || k < 0) {
-	/* Erroneous command. */
-	compsize = 0;
-    } else {
-	compsize = order * k;
+        /* Erroneous command. */
+        compsize = 0;
+    }
+    else {
+        compsize = order * k;
     }
     __GLX_SWAP_FLOAT_ARRAY(points, compsize);
 
 }
 
-void __glXDispSwap_Map2f(GLbyte *pc)
+void
+__glXDispSwap_Map2f(GLbyte * pc)
 {
     GLint uorder, vorder, ustride, vstride, k;
     GLfloat u1, u2, v1, v2, *points;
     GLenum target;
     GLint compsize;
+
     __GLX_DECLARE_SWAP_VARIABLES;
     __GLX_DECLARE_SWAP_ARRAY_VARIABLES;
 
@@ -109,35 +115,38 @@ void __glXDispSwap_Map2f(GLbyte *pc)
     __GLX_SWAP_FLOAT(pc + 8);
     __GLX_SWAP_FLOAT(pc + 16);
     __GLX_SWAP_FLOAT(pc + 20);
-    
-    target = *(GLenum *)(pc + 0); 
-    uorder = *(GLint *)(pc + 12);
-    vorder = *(GLint *)(pc + 24);
-    u1 = *(GLfloat *)(pc + 4);
-    u2 = *(GLfloat *)(pc + 8);
-    v1 = *(GLfloat *)(pc + 16);
-    v2 = *(GLfloat *)(pc + 20);
-    points = (GLfloat *)(pc + 28);
+
+    target = *(GLenum *) (pc + 0);
+    uorder = *(GLint *) (pc + 12);
+    vorder = *(GLint *) (pc + 24);
+    u1 = *(GLfloat *) (pc + 4);
+    u2 = *(GLfloat *) (pc + 8);
+    v1 = *(GLfloat *) (pc + 16);
+    v2 = *(GLfloat *) (pc + 20);
+    points = (GLfloat *) (pc + 28);
 
     k = __glEvalComputeK(target);
     ustride = vorder * k;
     vstride = k;
 
     if (vorder <= 0 || uorder <= 0 || k < 0) {
-	/* Erroneous command. */
-	compsize = 0;
-    } else {
-	compsize = uorder * vorder * k;
+        /* Erroneous command. */
+        compsize = 0;
+    }
+    else {
+        compsize = uorder * vorder * k;
     }
     __GLX_SWAP_FLOAT_ARRAY(points, compsize);
 
 }
 
-void __glXDispSwap_Map1d(GLbyte *pc)
+void
+__glXDispSwap_Map1d(GLbyte * pc)
 {
     GLint order, k, compsize;
     GLenum target;
     GLdouble u1, u2, *points;
+
     __GLX_DECLARE_SWAP_VARIABLES;
     __GLX_DECLARE_SWAP_ARRAY_VARIABLES;
 
@@ -146,41 +155,45 @@ void __glXDispSwap_Map1d(GLbyte *pc)
     __GLX_SWAP_INT(pc + 16);
     __GLX_SWAP_INT(pc + 20);
 
-    target = *(GLenum*) (pc + 16);
-    order = *(GLint*) (pc + 20);
+    target = *(GLenum *) (pc + 16);
+    order = *(GLint *) (pc + 20);
     k = __glEvalComputeK(target);
     if (order <= 0 || k < 0) {
-	/* Erroneous command. */
-	compsize = 0;
-    } else {
-	compsize = order * k;
+        /* Erroneous command. */
+        compsize = 0;
     }
-    __GLX_GET_DOUBLE(u1,pc);
-    __GLX_GET_DOUBLE(u2,pc+8);
-    __GLX_SWAP_DOUBLE_ARRAY(pc+24, compsize);
+    else {
+        compsize = order * k;
+    }
+    __GLX_GET_DOUBLE(u1, pc);
+    __GLX_GET_DOUBLE(u2, pc + 8);
+    __GLX_SWAP_DOUBLE_ARRAY(pc + 24, compsize);
     pc += 24;
 
 #ifdef __GLX_ALIGN64
-    if (((unsigned long)pc) & 7) {
-	/*
-	** Copy the doubles up 4 bytes, trashing the command but aligning
-	** the data in the process
-	*/
-	__GLX_MEM_COPY(pc-4, pc, compsize*8);
-	points = (GLdouble*) (pc - 4);
-    } else {
-	points = (GLdouble*) pc;
+    if (((unsigned long) pc) & 7) {
+        /*
+         ** Copy the doubles up 4 bytes, trashing the command but aligning
+         ** the data in the process
+         */
+        __GLX_MEM_COPY(pc - 4, pc, compsize * 8);
+        points = (GLdouble *) (pc - 4);
+    }
+    else {
+        points = (GLdouble *) pc;
     }
 #else
-    points = (GLdouble*) pc;
+    points = (GLdouble *) pc;
 #endif
 }
 
-void __glXDispSwap_Map2d(GLbyte *pc)
+void
+__glXDispSwap_Map2d(GLbyte * pc)
 {
     GLdouble u1, u2, v1, v2, *points;
     GLint uorder, vorder, ustride, vstride, k, compsize;
     GLenum target;
+
     __GLX_DECLARE_SWAP_VARIABLES;
     __GLX_DECLARE_SWAP_ARRAY_VARIABLES;
 
@@ -192,80 +205,86 @@ void __glXDispSwap_Map2d(GLbyte *pc)
     __GLX_SWAP_INT(pc + 36);
     __GLX_SWAP_INT(pc + 40);
 
-    target = *(GLenum *)(pc + 32);
-    uorder = *(GLint *)(pc + 36);
-    vorder = *(GLint *)(pc + 40);
+    target = *(GLenum *) (pc + 32);
+    uorder = *(GLint *) (pc + 36);
+    vorder = *(GLint *) (pc + 40);
     k = __glEvalComputeK(target);
     if (vorder <= 0 || uorder <= 0 || k < 0) {
-	/* Erroneous command. */
-	compsize = 0;
-    } else {
-	compsize = uorder * vorder * k;
+        /* Erroneous command. */
+        compsize = 0;
     }
-    __GLX_GET_DOUBLE(u1,pc);
-    __GLX_GET_DOUBLE(u2,pc+8);
-    __GLX_GET_DOUBLE(v1,pc+16);
-    __GLX_GET_DOUBLE(v2,pc+24);
-    __GLX_SWAP_DOUBLE_ARRAY(pc+44, compsize);
+    else {
+        compsize = uorder * vorder * k;
+    }
+    __GLX_GET_DOUBLE(u1, pc);
+    __GLX_GET_DOUBLE(u2, pc + 8);
+    __GLX_GET_DOUBLE(v1, pc + 16);
+    __GLX_GET_DOUBLE(v2, pc + 24);
+    __GLX_SWAP_DOUBLE_ARRAY(pc + 44, compsize);
     pc += 44;
     ustride = vorder * k;
     vstride = k;
 
 #ifdef __GLX_ALIGN64
-    if (((unsigned long)pc) & 7) {
-	/*
-	** Copy the doubles up 4 bytes, trashing the command but aligning
-	** the data in the process
-	*/
-	__GLX_MEM_COPY(pc-4, pc, compsize*8);
-	points = (GLdouble*) (pc - 4);
-    } else {
-	points = (GLdouble*) pc;
+    if (((unsigned long) pc) & 7) {
+        /*
+         ** Copy the doubles up 4 bytes, trashing the command but aligning
+         ** the data in the process
+         */
+        __GLX_MEM_COPY(pc - 4, pc, compsize * 8);
+        points = (GLdouble *) (pc - 4);
+    }
+    else {
+        points = (GLdouble *) pc;
     }
 #else
-    points = (GLdouble*) pc;
+    points = (GLdouble *) pc;
 #endif
 }
 
-void __glXDispSwap_CallLists(GLbyte *pc)
+void
+__glXDispSwap_CallLists(GLbyte * pc)
 {
     GLenum type;
     GLsizei n;
+
     __GLX_DECLARE_SWAP_VARIABLES;
     __GLX_DECLARE_SWAP_ARRAY_VARIABLES;
 
     __GLX_SWAP_INT(pc + 4);
     __GLX_SWAP_INT(pc + 0);
-    type = *(GLenum *)(pc + 4);
-    n = *(GLsizei *)(pc + 0);
+    type = *(GLenum *) (pc + 4);
+    n = *(GLsizei *) (pc + 0);
 
     switch (type) {
-      case GL_BYTE:
-      case GL_UNSIGNED_BYTE:
-      case GL_2_BYTES:
-      case GL_3_BYTES:
-      case GL_4_BYTES:
-	break;
-      case GL_SHORT:
-      case GL_UNSIGNED_SHORT:
-	__GLX_SWAP_SHORT_ARRAY(pc+8, n);
-	break;
-      case GL_INT:
-      case GL_UNSIGNED_INT:
-	__GLX_SWAP_INT_ARRAY(pc+8, n);
-	break;
-      case GL_FLOAT:
-	__GLX_SWAP_FLOAT_ARRAY(pc+8, n);
-	break;
+    case GL_BYTE:
+    case GL_UNSIGNED_BYTE:
+    case GL_2_BYTES:
+    case GL_3_BYTES:
+    case GL_4_BYTES:
+        break;
+    case GL_SHORT:
+    case GL_UNSIGNED_SHORT:
+        __GLX_SWAP_SHORT_ARRAY(pc + 8, n);
+        break;
+    case GL_INT:
+    case GL_UNSIGNED_INT:
+        __GLX_SWAP_INT_ARRAY(pc + 8, n);
+        break;
+    case GL_FLOAT:
+        __GLX_SWAP_FLOAT_ARRAY(pc + 8, n);
+        break;
     }
 
 }
 
-void __glXDispSwap_DrawArrays(GLbyte *pc)
+void
+__glXDispSwap_DrawArrays(GLbyte * pc)
 {
-    __GLXdispatchDrawArraysHeader *hdr = (__GLXdispatchDrawArraysHeader *)pc;
+    __GLXdispatchDrawArraysHeader *hdr = (__GLXdispatchDrawArraysHeader *) pc;
     __GLXdispatchDrawArraysComponentHeader *compHeader;
     int i;
+
     __GLX_DECLARE_SWAP_VARIABLES;
 
     __GLX_SWAP_INT(&hdr->numVertexes);
@@ -276,10 +295,10 @@ void __glXDispSwap_DrawArrays(GLbyte *pc)
     compHeader = (__GLXdispatchDrawArraysComponentHeader *) pc;
 
     /* compute stride (same for all component arrays) */
-    for (i=0; i<hdr->numComponents; i++) {
-	__GLX_SWAP_INT(&compHeader[i].datatype);
-	__GLX_SWAP_INT(&compHeader[i].numVals);
-	__GLX_SWAP_INT(&compHeader[i].component);
+    for (i = 0; i < hdr->numComponents; i++) {
+        __GLX_SWAP_INT(&compHeader[i].datatype);
+        __GLX_SWAP_INT(&compHeader[i].numVals);
+        __GLX_SWAP_INT(&compHeader[i].component);
 
     }
 
